﻿Imports DomainModel
Imports IdeaBlade.EntityModel

Namespace WpfApplication1

  ' Sample Push client.
  ' This sample will subscribe to two Push services:
  '   1) "LongRunningProcedure" - a service which sends period notifications to all its subscribers
  '   2) "OrderWatcher" - a service which sends notifications to subscribers based upon subscriber-specific arguments.


  Partial Public Class MainWindow
	  Inherits Window

	Private _entityManager As NorthwindIBEntities
	Private _owToken As String

	Public Sub New()
	  InitializeComponent()
	  AddHandler Loaded, AddressOf MainWindow_Loaded
	End Sub

	Private Sub MainWindow_Loaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  ' When connecting to a distributed BOS, it's always a good idea
	  ' to set up an error handler and issue an explicit Connect.
	  _entityManager = New NorthwindIBEntities(False)
	  AddHandler _entityManager.EntityServerError, Sub(o, args) MessageBox.Show(args.Exception.Message)
	  _entityManager.Connect()
	End Sub

	Private Sub btnSubscribeLRP_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  SubscribeLRP()
	End Sub

	Private Sub btnSubscribeOW_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  SubscribeOW()
	End Sub

	Private Sub btnUnsubscribeOW_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
	  UnsubscribeOW()
	End Sub

	#Region "LongRunningProcedure"
	' Subscribe to a LongRunningProcedure, which provides periodic status notifications.
	' When complete, we automatically unsubscribe from it.
	' Note that the RegisterCallback is executed asynchronously.  If it fails, the error
	' is passed to the callback.

	Private Sub SubscribeLRP()

	  ' Register for the service.
	  _entityManager.RegisterCallback("DomainModel.LongRunningProcedure, DomainModel", "MultiStepService", AddressOf LongRunningProcedureNotification, "lrp") ' user token -  the callback -  Method name -  Type name

	  Me.btnSubscribeLRP.IsEnabled = False

	  MessageBox.Show("You are now subscribed to the LongRunningProcedure.")
	End Sub

	Private Sub LongRunningProcedureNotification(ByVal op As SubscriptionOperation)
	  If op.HasError Then
		MessageBox.Show(op.Error.Message, "Subscription Error")
		Me.btnSubscribeLRP.IsEnabled = False
		Return
	  End If

	  ' This service returns one argument.
	  Dim info As LongRunningInfo = TryCast(op.NotificationData(0), LongRunningInfo)
	  Dim msg As String = info.Message
	  Dim workComplete As Integer = info.CompletionPercent

	  MessageBox.Show(msg, "Received from LongRunningProcedure")

	  ' We can unsubscribe once the procedure completes.
	  If workComplete = 100 Then
		op.CancelCallback()
		Me.btnSubscribeLRP.IsEnabled = True
	  End If
	End Sub
	#End Region


	#Region "OrderWatcher subscribe/unsubscribe"

	' Remember to add orders (manually or through another program) to
	' have OrderWatcher broadcast messages.  This will do:
	'    insert into [Order](customerid, employeeid, orderdate, rowversion)
	'    select customerid, 1, getdate(), 1 from customer where CompanyName = 'QUICK-Stop'

	Private Sub SubscribeOW()
	  ' The server-side code to be run.
      Dim serverDel As New ServerNotifyDelegate(AddressOf DomainModel.OrderWatcher.NewOrderService)

	  _owToken = "2" ' Arbitrary user token - we'll use this to unsubscribe.

	  Dim employeeId As Integer = 1 ' We'll pass this as a "client argument" into the service.

	  ' Register
	  _entityManager.RegisterCallback(serverDel, AddressOf OrderPushCallback, _owToken, employeeId) ' Client argument passed into the service -  Token identifying this subscription -  the callback action -  The service method

	  ' This example contains only a single subscription to this service from this client.  You
	  ' can subscribe to this or other services multiple times - each subscription
	  ' needs to use a unique UserToken.
	  Me.btnUnsubscribeOW.IsEnabled = True
	  Me.btnSubscribeOW.IsEnabled = False

	  MessageBox.Show("You are now subscribed to the OrderWatcher.")
	End Sub

	Private Sub UnsubscribeOW()
	  ' Cancel based on the UserToken we subscribed with.
	  _entityManager.CancelCallback(_owToken)

	  Me.btnUnsubscribeOW.IsEnabled = False
	  Me.btnSubscribeOW.IsEnabled = True
	  MessageBox.Show("You are now unsubscribed from the OrderWatcher.")
	End Sub

	Private Sub OrderPushCallback(ByVal op As SubscriptionOperation)
	  ' We know the service is returning a single message string...
	  Dim msg As String = op.NotificationData(0).ToString()
	  MessageBox.Show(msg, "Received from OrderWatcher")
	End Sub

	#End Region
  End Class
End Namespace
